####################################################################################################
# ファイル名：Interface.pm
# 機能：
# 管理GUIから、更新検知ツールを呼び出すためのインターフェース
# 
# 改訂履歴：
# 1.0.0：新規作成	2009/11/30
# 
####################################################################################################

package Interface;

# モジュール
use strict;
use warnings;
use lib qw(/var/detect/detect_update/pm);	# detect_update フォルダ(更新検知ツール)下の
											# pmフォルダを、絶対パス指定してください。
use XML::TreePP;
use Tie::IxHash;
use Encode;
use utf8;
use encoding "utf8";
use open IN  => ":utf8";
use open OUT => ":utf8";
use open IO => ":utf8";


# 自作モジュール
use DetectUpdate;
use Item;
#use SubItem;
#use MailTransfer;

####################################################################################################
# 関数名：detect_update
# 機能 ：引数で受けたアイテム番号のアイテムの、更新を検知する。
# 戻り値：1またはundef
####################################################################################################

sub detect_update {

	my $item_id = shift @_;

	# アイテム番号でレコードを取得
	my $item = Item->retrieve($item_id);


	# レコードから各項目を取得
	my $item_name = $item->item_name;
	my $item_url = $item->item_URL;
	my $follow_link = $item->follow_link;
	my $user_id = $item->user_id;
	my $password = $item->password;
	my $parameter = $item->parameter;

	# 項目をハッシュに格納
	my %Item_Infomation_h = (

		'ItemId' => $item_id, 
		'URL' => $item_url, 
		'Link' => $follow_link, 
		'UserId' => $user_id, 
		'PassWord' => $password, 
		'Parameter' => $parameter

	);

	# アイテムの更新を検知
	my $item_update = DetectUpdate::subItem(\%Item_Infomation_h);

	if ( defined($item_update) ) {	# 更新が検知されたら真

		# 更新情報を加工する変数
		my %Item_update_h;
		my %Update_Report_h;
		my %Item_Info_h;
		my @Results_l = ();
		my $xml = ();

		# ハッシュが順番を持つようにする
		tie %Item_update_h, 'Tie::IxHash';
		tie %Update_Report_h, 'Tie::IxHash';
		tie %Item_Info_h, 'Tie::IxHash';

		# 現在時刻を取得
		my $time = shift @$item_update;
		my %time = %$time;
		my $date = "$time{'Year'}/$time{'Month'}/$time{'Day'}($time{'Wday'}) $time{'Hour'}:$time{'Minute'}:$time{'Second'}";


		# XML出力

		my $treepp = XML::TreePP->new(use_ixhash => 1);

		my $xml_file = $item_id.'.xml';		# XMLファイル名

		# ファイルが存在する場合、Update要素を追加
		if ( -e DetectUpdate::UPDATE_REPORTS_DIR.'/'.$xml_file ) {

			my $tree = $treepp->parsefile( DetectUpdate::UPDATE_REPORTS_DIR.'/'.$xml_file );

			my $item_id_element = $tree->{'Update_Report'}->{'ItemInfo'}->{'Item_Id'};
			my $item_url_element = $tree->{'Update_Report'}->{'ItemInfo'}->{'Item_URL'};
			my $item_name_element = $tree->{'Update_Report'}->{'ItemInfo'}->{'Item_Name'};

			my $results = $tree->{'Update_Report'}->{'Results'};

			my $reffrence_type = ref($results);

			if ($reffrence_type eq 'ARRAY') {

				@Results_l = @$results;

				push ( @Results_l, { -Date => $date, Content => $item_update } );

			} elsif ($reffrence_type eq 'HASH') {

				push ( @Results_l, $results->{'Update'} );
				push ( @Results_l, { -Date => $date, Content => $item_update } );

			}

			%Item_Info_h = ( Item_Id => $item_id_element, Item_Name => $item_name_element, Item_URL => $item_url_element );

			%Item_update_h  = ( ItemInfo=>\%Item_Info_h, Results=>{ Update=>[@Results_l] } );

			%Update_Report_h = ( Update_Report => \%Item_update_h );

			$xml = $treepp->write( \%Update_Report_h );


		# XMLファイルが存在しない場合
		} else {

			push ( @Results_l, { -Date => $date, Content => $item_update } );

			my $data = $item->item_name;
#			print $data;


			%Item_Info_h = ( Item_Id => $item->item_id, Item_Name => $item->item_name, Item_URL => $item_url );

			%Item_update_h = ( ItemInfo=>\%Item_Info_h, Results=>{ Update=>[@Results_l] } );

			%Update_Report_h = ( Update_Report => \%Item_update_h );
			$xml = $treepp->write( \%Update_Report_h );

		}


		# ファイルへ書き出す
		open (XML, '+>'.DetectUpdate::UPDATE_REPORTS_DIR.'/'.$xml_file);
		print XML $xml;
		close (XML);

		my $chmod_target = DetectUpdate::UPDATE_REPORTS_DIR.'/'.$xml_file;
		open (CHMOD, "chmod 777 $chmod_target |");
		close(CHMOD);



		# 更新検知された場合、1を返す
		return 1;


	} else {

		# 更新検知されなかった場合、undefを返す
		return undef;

	}



}

1;
