#!/usr/bin/perl
########################################################################################
# ファイル名：exec_group_update.cgi
# 機能：アイテムグループの編集登録または新規登録
#
# 改訂履歴：
# 1.0.0 ：新規作成      2009/11/19  M.Motomura
# 1.0.1 ：修正　INSERT、UPDATE文実行失敗時のロールバック処理の追加
#　　　　　　　　　　　　　　　　　　　　　　2010/6/4　M.Motomura
########################################################################################
use strict;
use warnings;
use lib qw( ./pm );
use system_config;
use CGI;
use common;
use HTML::Template;
use utf8;

&main();
exit(1);

########################################################################################
# 関数名:main
# 機能  :main処理
# 引数  :なし
# 戻り値:1
########################################################################################
sub main {
	# CGIオブジェクト
	my $query = new CGI;
	
	&common::check_referer($query);
	
	my($result_comment);
	
	# 引数の取得
	my $group_name = $query->param('group_name');
	my $group_id = $query->param('group_id');
	my $comment = $query->param('comment');
	
	# アイテムグループが登録されているか確認
	my $cnt = 0;
	unless ($group_id eq 'new') {
		$cnt = &count_group($group_id, $group_name, $comment);
	}
	
	if($cnt == 0) {
		# DB接続
		my $dbh = &common::db_open();
		
		if($group_id eq 'new') {
			&tbl_item_group_insert($dbh, $group_name, $comment);
		} else {
			&tbl_item_group_update($dbh, $group_id, $group_name, $comment);
		}
		
		$result_comment = $::msg_update_finish;
	} else {
		$result_comment = $::msg_same_data;
	}
	# ----------------------------------
	# テンプレートのパラメータ設定
	#
	my $template = HTML::Template->new(filename => './template/group_update.tmpl');
	$template->param( COMMENT => $result_comment );

	# 返値設定
	print $query->header(-type=>'text/html', -charset=>'utf-8');
	print $template->output;
	
	return(1);
}

########################################################################################
# 関数名:count_group
# 機能  :指定されたグループ名が登録されているか確認
# 引数  :$group_name
# 戻り値:$cnt(0→更新対象データ　1→グループ名重複(アップデート対象外))
########################################################################################
sub count_group {
	my($group_id);
	my($group_name);
	my($comment);
	my($cnt);
	my($sth);
	
	# 引数取得
	$group_id = shift(@_);
	$group_name = shift(@_);
	$comment = shift(@_);
	
	my $dbh = &common::db_open();

	# 同一レコードかの確認(更新項目有無の確認)
	my $sql = qq { SELECT item_group_id FROM tbl_item_group WHERE item_group_id = ?
				AND item_group_name = ? AND comment = ? };
	my @data = ($group_id, $group_name, $comment);
	
	($cnt, $sth) = &common::count_sql_result($dbh, $sql, 'exec_group_update.cgi', @data);
	
	# 検索終了(SQL開放)
	#########################################
	$sth->finish;
	
	# コミット＆切断
	#########################################
	$dbh->commit || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	
	return($cnt);
}

########################################################################################
# 関数名:tbl_item_group_insert
# 機能  :グループアイテムの新規登録
# 引数  :$dbh, $group_name, $comment
# 戻り値:1
########################################################################################
sub tbl_item_group_insert {
	my($sth);
	
	my ($dbh, $group_name, $comment) = @_;
	
	my $sql = qq { INSERT INTO tbl_item_group (item_group_name, comment) 
				 VALUES(?, ?)};
	my @data = ($group_name, $comment);
	
	$sth = &common::execute_sql($dbh, $sql, 'exec_group_update.cgi', 
					'./template/group_update.tmpl', 'COMMENT', $::msg_db_error1, @data);

	# 検索終了(SQL開放)
	#########################################
	$sth->finish;
	
	# コミット or ロールバック ＆切断
	#########################################
	$dbh->commit || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	
	return(1);
}

########################################################################################
# 関数名:tbl_item_group_update
# 機能  :グループアイテムの更新登録
# 引数  :$dbh, $group_id, $group_name, $comment
# 戻り値:1
########################################################################################
sub tbl_item_group_update {
	my($sth);
	
	my ($dbh, $group_id, $group_name, $comment) = @_;
	
	my $u_sql = qq { UPDATE  tbl_item_group SET item_group_name = ?, comment = ?
				WHERE item_group_id = ?};
	my @data = ($group_name, $comment, $group_id);
	
	$sth = &common::execute_sql($dbh, $u_sql, 'exec_group_update.cgi', 
					'./template/group_update.tmpl', 'COMMENT', $::msg_db_error1, @data);
	
	# 検索終了(SQL開放)
	#########################################
	$sth->finish;
	
	# コミット or ロールバック ＆切断
	#########################################
	$dbh->commit || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	
	return(1);
}
