#!/usr/bin/perl
########################################################################################
# ファイル名：search_update_histry.cgi
# 機能：選択されたアイテムの更新履歴を取得する。
#
# 改訂履歴：
# 1.0.0 ：新規作成      2009/11/12
# 1.0.1 ：バグ修正　更新履歴のURL取得方法の変更
#　　　　日付ディレクトリの下のlink.txtファイルの中身を読み込んでURLに付加する
#　　　　(数字)<tab>(URLパス)　(数字)が2の場合はURLパスに".html"を付加したURLの
#　　　　2つを表示する　2010.6.4
########################################################################################
use strict;
use warnings;
use lib qw( ./pm );
use system_config;
use CGI;
use common;
use HTML::Template;
use File::Basename;
use utf8;
&main();
exit(1);

########################################################################################
# 関数名:main
# 機能  :main処理
# 引数  :なし
# 戻り値:1
########################################################################################

sub main {
	my $query = new CGI;
	
	# ----------------------------------
	# パラメータ取得　検索キー
	my $item		= $query->param("item");
	my $item_id		= $query->param("item_id");
	my $url			= $query->param("url");
	my $start_date	= $query->param("start");
	my $end_date	= $query->param("end");
	
	my $start = &common::check_date($start_date);
	my $end   = &common::check_date($end_date);
	
	# ----------------------------------
	# ディレクトリ検索設定
	my ($history, $error, $exist) = &get_update_histry($item, $item_id, $url, $start, $end);
	# ----------------------------------
	# テンプレートのパラメータ設定
	my $template;
	if ($exist == 1 ) {
		$template = HTML::Template->new(filename => './template/update_histry_info.tmpl', 
										die_on_bad_params => 0,
										global_vars       => 1,
										loop_context_vars => 1);
		$template->param( HISTORY => $history );
		$template->param( ERROR   => $error );
		
		print $query->header(-type=>'text/html', -charset=>'utf-8');
		print $template->output;
	} else {
		my $message = ($error == 1 ? $::msg_category_error : $::msg_history_not_found);
		
		print $query->header(-type=>'text/plane', -status=>'404 Not Found', -charset=>'utf-8');
		print $message;
	}

	return(1);
}

########################################################################################
# 関数名:get_update_histry
# 機能  :ソート後の更新履歴エリアに表示する内容一覧
# 引数  :$item, $id, $url, $start, $end
# 戻り値:$history	更新履歴のリスト
#		 $error		カテゴリコードの対応が取れていないものがあるかのフラグ
#		 $exist		$historyに要素があるかのフラグ
########################################################################################
sub get_update_histry {
	my ($item, $id, $url, $start, $end) = @_;
	
	my $dbh = &common::db_open();
	
	# tbl_category_codeからデータを取得
	my $category_list_sql = "select code, name from tbl_category_code order by code asc";
	
	my ($category_list_count, $category_list_sth)
		= &common::count_sql_result($dbh, $category_list_sql, 'search_update_histry.cgi');
	
	return (&error_finish($dbh, 1, $category_list_sth)) if ($category_list_count == 0);
	
	my @category_code;

	while(my @category_list_result = $category_list_sth->fetchrow) {
		my %category_list_data;
		$category_list_data{code} = $category_list_result[0];
		$category_list_data{name} = $category_list_result[1];

		push(@category_code, \%category_list_data);
	}

	# tbl_update_histryから更新履歴を取得
	my $update_histry_sql;
	my @update_histry_data;
	
	if ($start != 0 && $end != 0) {
		$update_histry_sql  = "select id, date_update, time_update from tbl_update_histry
								where item_id = ? and date_update >= ? and date_update <=?";
		@update_histry_data = ($id, $start, $end);
	}
	
	if ($start != 0 && $end == 0) {
		$update_histry_sql  = "select id, date_update, time_update from tbl_update_histry
								where item_id = ? and date_update >= ?";
		@update_histry_data = ($id, $start);
	}
	
	if ($start == 0 && $end != 0) {
		$update_histry_sql  = "select id, date_update, time_update from tbl_update_histry
								where item_id = ? and date_update <= ?";
		@update_histry_data = ($id, $end);
	}
	
	if ($start == 0 && $end == 0) {
		$update_histry_sql  = "select id, date_update, time_update from tbl_update_histry
								where item_id = ?";
		@update_histry_data = ($id);
	}
	
	my ($update_histry_count, $update_histry_sth)
		= &common::count_sql_result($dbh, $update_histry_sql, 'search_update_histry.cgi', @update_histry_data);
	
	return (&error_finish($dbh, 0, $update_histry_sth)) if ($update_histry_count == 0);
	
	my @sql_result;

	while(my @update_histry_result = $update_histry_sth->fetchrow) {
		my %update_histry_data;
		$update_histry_data{history_id}  = $update_histry_result[0];
		$update_histry_data{date_update} = $update_histry_result[1];
		$update_histry_data{time_update} = $update_histry_result[2];

		push(@sql_result, \%update_histry_data);
	}
	
	# idに対応したディレクトリを開く
	my $dh;
	my $dirname = "$::dir_histry/$id";
	
	return (&error_finish($dbh, 0)) unless (opendir($dh, $dirname));
	
	my @dir_list = grep {/\d{4}-\d{2}-\d{2}_\d{2}-\d{2}-\d{2}/} readdir ($dh);
	
	my @history_list;
	my $not_found_category = 0;		# tbl_category_code未登録code発見フラグ
	
	foreach my $link_data (sort {$b cmp $a} @dir_list) {
		# ディレクトリ名からページ取得日を取得
		my @date_list = split(/_/, $link_data);
		my $dir_date  = $date_list[0];
		my $dir_time  = $date_list[1];
		
		# tbl_update_histryから得た日付・時刻とディレクトリ名の一致
		foreach my $history (@sql_result) {
			my %history  = %$history;
			my $sql_date = $history{date_update};
			my $sql_time = $history{time_update};
			
			$sql_date =~ s/^(\d{4})(\d{2})(\d{2})$/$1-$2-$3/;
			$sql_time =~ s/^(\d{2})(\d{2})(\d{2})$/$1-$2-$3/;
			
			next unless ($dir_date eq $sql_date && $dir_time eq $sql_time);
			
			# リファレンス先を変更して初期化
			my @category;
			foreach my $ref_category (@category_code) {
				my %new_category = %$ref_category;
				push (@category, \%new_category);
			}
			
			my $history_category_sql  = "select category_code from tbl_category
										where history_id = ? order by category_code asc";
			my @history_category_data = ($history{history_id});
			
			my ($history_category_count, $history_category_sth)
				= &common::count_sql_result($dbh, $history_category_sql, 'search_update_histry.cgi', 
											@history_category_data);
			
			# 更新カテゴリの設定
			if ($history_category_count == 0) {
				$category[0]{checked} = 1;
			} else {
				my @checked_category;
				
				while(my @history_category_result = $history_category_sth->fetchrow) {
					my %checked_category;
					$checked_category{code} = $history_category_result[0];
					
					push(@checked_category, \%checked_category);
				}
				
				my $not_found_count = 0;
				
				# 更新履歴カテゴリを設定
				foreach my $ref_checked_category (@checked_category) {
					my %checked_category = %$ref_checked_category;
					
					my $not_found_flag   = 1;
					foreach my $ref_database_category (@category) {
						my %database_category = %$ref_database_category;
						
						if ($checked_category{code} eq $database_category{code}) {
							$$ref_checked_category{checked}  = 1;
							$$ref_database_category{checked} = 1;
							$not_found_flag				= 0;
							last;
						}
					}
					$not_found_count++ if ($not_found_flag == 1);
				}
				
				$category[0]{checked} = 1 if ($not_found_count == @checked_category);
				
				# tbl_category_codeに一致しないデータを警告
				if ($not_found_count) {
					$not_found_category = 1;
					
					for (my $i = 0; $i < @checked_category; $i++) {
						warn "category_code $checked_category[$i]{code} is not found\n"
							unless (defined($checked_category[$i]{checked}));
					}
				}
			}
			
			# リンク先の取得
			my $page_url  = qq {$::histry_url/$id/$link_data/};
			
			my $file_text = &read_path_file("$dirname/$link_data");
			my @list      = split(/\t/, $file_text);

			my ($file, $path)  = fileparse($list[1]);
			my $diff_html_url  = "$page_url$path" . "diff.html";
			my $diff_txt_url   = "$page_url$path" . "diff.txt";
			my $diff_html_file = "$dirname/$link_data/$path" . "diff.html";
			my $diff_txt_file  = "$dirname/$link_data/$path" . "diff.txt";

			$page_url    .= $list[1];

			my %data;
			
			$data{link_date}  = $sql_date."_".$sql_time;
			$data{history_id} = $history{history_id};
			$data{category}   = \@category;
			$data{first_url}  = qq{$page_url};
			$data{second_url} = qq{$page_url.html} if ($list[0] == 2);
			$data{diff_html}  = $diff_html_url if (-e $diff_html_file);
			$data{diff_txt}   = $diff_txt_url  if (-e $diff_txt_file); 
			
			push (@history_list, \%data);
		}
	}
	closedir $dh;
	
	$dbh->commit     || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	
	return $#history_list >= 0 ? (\@history_list, $not_found_category, 1) : (\@history_list, 0, 0);
}

sub error_finish {
	my ($dbh, $error, $sth) = @_;
	
	$sth->finish if (defined($sth));
	
	$dbh->rollback   || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	
	return ("", $error, 0);
}

########################################################################################
# 関数名:read_path_file
# 機能  :更新履歴へのパスが書かれたファイルを読み込む
# 引数  :PATH FILE_NAME
# 戻り値:ファイルに書かれているテキスト
########################################################################################
sub read_path_file {
	my($path) = shift(@_);
	open(FH, "$path/$::histry_file_path");
	my $str = <FH>;
	chomp($str);
	close(FH);
	return $str;
}

########################################################################################
# 関数名:make_url =>不使用
# 機能  :ＵＲＬの作成
# 引数  :URL
# 戻り値:編集後URL
########################################################################################
sub make_url {
	my($str);
	my($path);
	my($second_path);
	
	# 引数受取
	$str = shift(@_);
	my @list = split(/\/\//, $str, 2);
	my $s_str = $list[1];
	chomp($s_str);
	
	if( $s_str =~ /(.+)\.html$/ || $s_str =~ /(.+)\.html\#(.+)/ ) {	# 拡張子が.htmlまたは.html#ラベル
		$path = $s_str;
# 20100604 yazawa start
	} elsif( $s_str =~ /(.+)\.html\?(.+)/ ) { # 拡張子が.html?パラメータまたは.html?パラメータ#ラベル
		$path = $1.'.html%3f'.$2.'.html';
# 20100604 yazawa end
	} elsif( $s_str =~ /(.+)\.html\?(.+)\#(.+)/ || $s_str =~ /(.+)\.html\?(.+)/ ) { # 拡張子が.html?パラメータまたは.html?パラメータ#ラベル
		$path = $1.'.html%3f'.$2;
	} elsif( $s_str =~ /(.+)\/$/ ) {								# /で終わるもの
		$path = $s_str;
	} elsif( $s_str =~ /(.+)\.htm\#(.+)/ ) {						# 拡張子が.htm#ラベル
		$path = $1.'.htm.html#'.$2;
	} elsif( $s_str =~ /(.+)\.htm$/ ) {								# 拡張子が.htm
		$path = $1.'.htm.html';
	} elsif($s_str =~ /(.+)\.htm\?(.+)\#(.+)/ ) {					# 拡張子が.htm?パラメータ#ラベル
		$path = $1.'.htm%3f'.$2.'.html#'.$3;
	} elsif($s_str =~ /(.+)\.htm\?(.+)/) {							# 拡張子が.htm?パラメータ
		$path = $1.'.htm%3f'.$2.'.html';
	} elsif( $s_str =~ /(.+)\.jsp\#(.+)/ ) {						# 拡張子が.jsp#ラベル
		$path = $1.'.jsp.html#'.$2;
	} elsif($s_str =~ /(.+)\.jsp$/) {								# 拡張子が.jsp
		$path = $1.'.jsp.html';
	} elsif($s_str =~ /(.+)\.jsp\?(.+)\#(.+)/ ) {					# 拡張子が.jsp?パラメータ#ラベル
		$path = $1.'.jsp%3f'.$2.'.html#'.$3;
	} elsif($s_str =~ /(.+)\.jsp\?(.+)/) {							# 拡張子が.jsp?パラメータ
		$path = $1.'.jsp%3f'.$2.'.html';
	} elsif( $s_str =~ /(.+)\.php\#(.+)/ ) {						# 拡張子が.php#ラベル
		$path = $1.'.php.html#'.$2;
	} elsif($s_str =~ /(.+)\.php$/) {								# 拡張子が.php
		$path = $1.'.php.html';
	} elsif($s_str =~ /(.+)\.php\?(.+)\#(.+)/ ) {					# 拡張子が.php?パラメータ#ラベル
		$path = $1.'.php%3f'.$2.'.html#'.$3;
	} elsif($s_str =~ /(.+)\.php\?(.+)/) {							# 拡張子が.php?パラメータ
		$path = $1.'.php%3f'.$2.'.html';
	} elsif($s_str =~ /(.+)\.cgi\#(.+)/ ) {							# 拡張子が.cgi#ラベル
		$path = $1.'.cgi.html#'.$2;
	} elsif($s_str =~ /(.+)\.cgi$/) {								# 拡張子が.cgi
		$path = $1.'.cgi.html';
	} elsif($s_str =~ /(.+)\.cgi\?(.+)\#(.+)/ ) {					# 拡張子が.cgi?パラメータ#ラベル
		$path = $1.'.cgi%3f'.$2.'.html#'.$3;
	} elsif($s_str =~ /(.+)\.cgi\?(.+)/) {							# 拡張子が.cgi?パラメータ
		$path = $1.'.cgi%3f'.$2.'.html';
	} elsif($s_str =~ /(.+)\.do\#(.+)/ ) {							# 拡張子が.do#ラベル
		$path = $1.'.do.html#'.$2;
	} elsif($s_str =~ /(.+)\.do$/) {								# 拡張子が.do
		$path = $1.'.do.html';
	} elsif($s_str =~ /(.+)\.do\?(.+)\#(.+)/ ) {					# 拡張子が.do?パラメータ#ラベル
		$path = $1.'.do%3f'.$2.'.html#'.$3;
	}elsif($s_str =~ /(.+)\.do\?(.+)/) {							# 拡張子が.do?パラメータ
		$path = $1.'.cgi%3f'.$2.'.html';
	} else {
		my $i = rindex($s_str, "\/" );
		my $param = substr($s_str, $i + 1, length($s_str)-1);
		my $front_param = substr($s_str, 0, $i);
		if( $param =~ /(.+)\.(.+)/ ) {								# 拡張子あり
			$path = $s_str;
		} else {
			if( $param =~ /(.+)\#(.+)/ ) {
				$path = $front_param.'/'.$1.'.html#'.$2;			# 拡張子なし#ラベル
			} else {
				$path = $s_str.'.html';								# 拡張子なし
				$second_path = $s_str;
			}
		}
	}
	return($path, $second_path);

}
